# -*- coding: utf-8 -*-
"""
Created on Wed Mar 11 18:00:28 2020

@author: gamelina
"""

import at
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d


class Optics:
    """Handle optic functions"""
    def __init__(self, lattice_file=None, local_beta=None, local_alpha=None, 
                 local_dispersion=None, **kwargs):
        
        if lattice_file is not None:
            self.use_local_values = False
            self.load_from_AT(lattice_file, **kwargs)
            if local_beta is None:
                self.local_beta = np.mean(self.beta_array)
            else:
                self.local_beta = local_beta
            if local_alpha is None:
                self.local_alpha = np.mean(self.alpha_array)
            else:
                self.local_alpha = local_alpha
            if local_dispersion is None:
                self.local_dispersion = np.zeros((4,))
            else:
                self.local_dispersion = local_dispersion
            self.local_gamma = (1 + self.local_alpha**2)/self.local_beta
            
        else:
            self.use_local_values = True
            self.local_beta = local_beta
            self.local_alpha = local_alpha
            self.local_gamma = (1 + self.local_alpha**2)/self.local_beta
            self.local_dispersion = local_dispersion

    def load_from_AT(self, lattice_file, **kwargs):
        
        self.n_points = int(kwargs.get("n_points", 1e3))
        periodicity = kwargs.get("periodicity")
        
        self.lattice = at.load_lattice(lattice_file)
        lattice = self.lattice.slice(slices=self.n_points)
        refpts = np.arange(0, len(lattice))
        twiss0, tune, chrom, twiss = at.get_twiss(lattice, refpts=refpts,
                                                  get_chrom=True)
        
        if periodicity is None:
            self.periodicity = lattice.periodicity
        else:
            self.periodicity = periodicity
                    
        for i in range(self.periodicity-1):
            pos = np.append(twiss.s_pos, twiss.s_pos + twiss.s_pos[-1]*(i+1))
            
        self.position = pos
        self.beta_array = np.tile(twiss.beta.T, self.periodicity)
        self.alpha_array = np.tile(twiss.alpha.T, self.periodicity)
        self.gamma_array = (1 + self.alpha_array**2)/self.beta_array
        self.dispersion_array = np.tile(twiss.dispersion.T, self.periodicity)
        self.tune = tune * self.periodicity
        self.chro = chrom * self.periodicity
        self.ac = at.get_mcf(self.lattice)
        
        self.setup_interpolation()
        
        
    def setup_interpolation(self):        
        self.betaX = interp1d(self.position, self.beta_array[0,:],
                              kind='linear')
        self.betaY = interp1d(self.position, self.beta_array[1,:],
                              kind='linear')
        self.alphaX = interp1d(self.position, self.alpha_array[0,:],
                               kind='linear')
        self.alphaY = interp1d(self.position, self.alpha_array[1,:],
                               kind='linear')
        self.gammaX = interp1d(self.position, self.gamma_array[0,:],
                               kind='linear')
        self.gammaY = interp1d(self.position, self.gamma_array[1,:],
                               kind='linear')
        self.dispX = interp1d(self.position, self.dispersion_array[0,:],
                              kind='linear')
        self.disppX = interp1d(self.position, self.dispersion_array[1,:],
                               kind='linear')
        self.dispY = interp1d(self.position, self.dispersion_array[2,:],
                              kind='linear')
        self.disppY = interp1d(self.position, self.dispersion_array[3,:],
                               kind='linear')
        
    def beta(self, position):
        if self.use_local_values:
            return np.outer(self.local_beta, np.ones_like(position))
        else:
            beta = [self.betaX(position), self.betaY(position)]
            return np.array(beta)
    
    def alpha(self, position):
        if self.use_local_values:
            return np.outer(self.local_alpha, np.ones_like(position))
        else:
            alpha = [self.alphaX(position), self.alphaY(position)]
            return np.array(alpha)
    
    def gamma(self, position):
        if self.use_local_values:
            return np.outer(self.local_gamma, np.ones_like(position))
        else:
            gamma = [self.gammaX(position), self.gammaY(position)]
            return np.array(gamma)
    
    def dispersion(self, position):
        if self.use_local_values:
            return np.outer(self.local_dispersion, np.ones_like(position))
        else:
            dispersion = [self.dispX(position), self.disppX(position), 
                          self.dispY(position), self.disppY(position)]
            return np.array(dispersion)
    

